rm(list=ls())
setwd(choose.dir())
source("IM_proc_eval.r")

#############################
##A/Performing the simulations
#############################

##1/Parameters definition

m<<-20#number of imputed data sets
S<<-2#number of underlying dimensions
nsim<<-1000#number of simulations
  
Parameters.temp<<-matrix(NA,16,4)
colnames(Parameters.temp)<-c("I","J","rho","%")
Parameters.temp[,1]<-c(rep(30,8),rep(200,8))
Parameters.temp[,2]<-rep(c(rep(6,4),rep(60,4)),2)
Parameters.temp[,3]<-rep(c(rep(.3,2),rep(.9,2)),4)
Parameters.temp[,4]<-rep(c(.1,.3),8)
Parameters<<-list()
for(case in 1:16){
  # configuration parameters
  I<-Parameters.temp[case,1]
  P<-Parameters.temp[case,2]
  sigma<-Parameters.temp[case,3]
  p<-Parameters.temp[case,4]
  
  # regression formulae for the beta coefficient
  formula<-concat(c("y~",paste(paste("x",1:(P-2),sep=""),"+",sep=""),"x",P-1))
  if(I<P){formula<-"y~x1"}
  
  # variance matrix to draw a data set
  mat<-diag(1,P)
  if (P==6){
    mat[1:4,1:4]<-sigma
    mat[5:6,5:6]<-sigma
  }else if(P==60){
    mat[1:40,1:40]<-sigma
    mat[41:60,41:60]<-sigma
  }
  diag(mat)<-1
  
  # competiting algorithms according to the configuration
  methods<-1:4
  names(methods)<-c("LD","amelia","BayesMI","BayesMIPCA")
  methods.temp<-methods
  no.amelia<-c(2,4:8,13:16)
  no.mice<-c(5:8)
  no.ld<-c(2,4:8,13:16)
  if (case %in% no.mice){methods.temp<-methods.temp[-3]}
  if (case %in% no.amelia){methods.temp<-methods.temp[-2]}
  if (case %in% no.ld){methods.temp<-methods.temp[-1]}
  
  Parameters[[case]]<-list(I=I,P=P,rho=sigma,p=p,mat=mat,formula=formula,methods=methods.temp)
}

##2/Performing all the simulations (Time consuming!)
SIMULATION<-function(case){
  # SIMULATION(1) perform simulation for case 1
  I<-Parameters[[case]][[1]]
  P<-Parameters[[case]][[2]]
  rho<-Parameters[[case]][[3]]
  p<-Parameters[[case]][[4]]
  cat(case,"I=",I,"J=",P,"rho=",rho,"%=",p,"\n")
  mat<-Parameters[[case]][[5]]
  formula<-Parameters[[case]][[6]]
  method<-Parameters[[case]][[7]]
  #simulation for the current case
  res<-simulate(nsim=nsim,p=p,v.na=c(1:P),Xtild=NULL,Sigma.create=mat,formula=formula,S=S,m=m,method=method,n=I,v1.cor=P,v2.cor=(P-1))
  assign(paste("res.article.",case,sep=""),res)
  save(list=paste("res.article.",case,sep=""),file=paste("res.article.",case,".Rdata",sep=""))
}

# for(case in c(1:16)){SIMULATION(case)}

#############################
##B/ Generating of the tables
#############################

##1/Confidence interval width for a complete data set
ci.comp<-matrix(NA,nrow=8,ncol=6)
ci.comp[,1:3]<-Parameters.temp[seq(1,16,2),1:3]
colnames(ci.comp)<-c("I","P","rho","mean","beta","cor")
for (case in 1:8){
  I<-Parameters[[(2*case-1)]][[1]]
  P<-Parameters[[(2*case-1)]][[2]]
  rho<-Parameters[[(2*case-1)]][[3]]
  p<-Parameters[[(2*case-1)]][[4]]
  mat<-Parameters[[(2*case-1)]][[5]]
  formula<-Parameters[[(2*case-1)]][[6]]
  method<-Parameters[[(2*case-1)]][[7]]
  l.mean<-l.beta<-l.cor<-c()
  for (i in 1:1000){
    set.seed(i)
    X<-createdata(n=I,Sigma=mat)   
    temp<-LD(X,formula=formula,P-1,P)
    l.mean<-c(l.mean,temp[["mean"]][["hi 95"]]-temp[["mean"]][["lo 95"]])
    l.beta<-c(l.beta,temp[["beta"]][["hi 95"]]-temp[["beta"]][["lo 95"]])
    l.cor<-c(l.cor,temp[["cor"]][["hi 95"]]-temp[["cor"]][["lo 95"]])
  }
  ci.comp[case,"mean"]<-median(l.mean)
  ci.comp[case,"beta"]<-median(l.beta)
  ci.comp[case,"cor"]<-median(l.cor)
}
ci.comp<-round(ci.comp,3)

#CI width for a complete data set for the expectation
print(ci.comp[c(1,5),c("I","mean")])

#CI width for a complete data set for the regression coefficient
print(ci.comp[-c(3,4),c("I","P","rho","beta")])

# CI width for a complete data set for the correlation coefficient
print(ci.comp[c(1,2,5,6),c("I","rho","cor")])

##2/Results for incomplete data set
T.res<-array(NA,dim=c(3,16,20),dimnames=list(c("beta","mean","cor"),
      paste("case",1:16),c("I","J","rho","%",paste(c("LD","Amelia","BayesMI","BayesMIPCA"),"bias"),
      paste(c("LD","Amelia","BayesMI","BayesMIPCA"),"rmse"),
      paste(c("LD","Amelia","BayesMI","BayesMIPCA"),"aiw"),
      paste(c("LD","Amelia","BayesMI","BayesMIPCA"),"cov"))))
T.res[1,,1:4]<-Parameters.temp
T.res[2,,1:4]<-Parameters.temp
T.res[3,,1:4]<-Parameters.temp

for (case in 1:16){
  I<-Parameters[[case]][[1]]
  P<-Parameters[[case]][[2]]
  rho<-Parameters[[case]][[3]]
  p<-Parameters[[case]][[4]]
  mat<-Parameters[[case]][[5]]
  method<-Parameters[[case]][[7]]
  
  #loading of the simulation results
  load(paste("res.article.",case,".Rdata",sep=""))
  get(paste("res.article.",case,sep=""))
  
  #sum up of the results
  res.sum<-summarize.results(get(paste("res.article.",case,sep="")),true_beta=(solve(mat[-1,-1])%*%mat[1,(2:P)])[1],true_mean=0,true_cor=rho,digits=3)                         
  numcol.bayespca<-seq(8,20,4);numcol.cca<-seq(5,17,4);numcol.bayesmi<-seq(7,19,4);numcol.amelia<-seq(6,18,4)
  if ("BayesMI" %in% names(method)){
    T.res["mean",case,numcol.bayesmi]<-res.sum$mean[c(1,4,2,3),"BayesMI"]
    if(I>P){T.res["beta",case,numcol.bayesmi]<-res.sum$beta[c(1,4,2,3),"BayesMI"]}
    T.res["cor",case,numcol.bayesmi]<-res.sum$cor[c(1,4,2,3),"BayesMI"]
  }
  if ("BayesMIPCA" %in% names(method)){
    T.res["mean",case,numcol.bayespca]<-res.sum$mean[c(1,4,2,3),"BayesMIPCA"]
    if(I>P){T.res["beta",case,numcol.bayespca]<-res.sum$beta[c(1,4,2,3),"BayesMIPCA"]}
    T.res["cor",case,numcol.bayespca]<-res.sum$cor[c(1,4,2,3),"BayesMIPCA"]
  }
  if ("amelia" %in% names(method)){
    T.res["mean",case,numcol.amelia]<-res.sum$mean[c(1,4,2,3),"amelia"]
    if(I>P){T.res["beta",case,numcol.amelia]<-res.sum$beta[c(1,4,2,3),"amelia"]}
    T.res["cor",case,numcol.amelia]<-res.sum$cor[c(1,4,2,3),"amelia"]
  }
  if ("LD" %in% names(method)){
    T.res["mean",case,numcol.cca]<-res.sum$mean[c(1,4,2,3),"LD"]
    if(I>P){T.res["beta",case,numcol.cca]<-res.sum$beta[c(1,4,2,3),"LD"]}
    T.res["cor",case,numcol.cca]<-res.sum$cor[c(1,4,2,3),"LD"]
  }
}

#Table 1
print(T.res["mean",,c(1:4,13:20)])

#Table 2
width.comp<-c(.659,.659,0.147,.147,.659,.659,0.147,.147,0.253,0.253, 0.053,0.053, 0.253,0.253, 0.053,0.053)#from 1/
ratio<-round(100*(sweep(as.matrix(T.res["cor",,13:16]),MARGIN=1,STATS=width.comp,FUN='/')-1),0)
colnames(ratio)<-paste(colnames(ratio),"ratio")
print(cbind(T.res["cor",,c(1:4)],ratio,T.res["cor",,c(17:20)]))

#Results for the regression coefficient
print(T.res["beta",-c(5:8),c(1:4,9:12)])
print(T.res["beta",-c(5:8),c(1:4,13:16)])
print(T.res["beta",-c(5:8),c(1:4,17:20)])
  
